﻿#include "qt-inventor.h"
#include "ui_qt-inventor.h"

#include <Inventor/Qt/SoQt.h>
#include <Inventor/Qt/viewers/SoQtExaminerViewer.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoMaterial.h>

class TreeItem : public QTreeWidgetItem
{
public:
    SoNode * node;
};

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    ui->splitter->setStretchFactor (0, 1);
    ui->splitter->setStretchFactor (1, 5);
    ui->splitter->setStretchFactor (2, 1);

    ui->prop->setColumnCount (2);
    ui->prop->setHeaderLabels (QStringList () << "name" << "value");

    SoQtExaminerViewer * examiner = new SoQtExaminerViewer (ui->widget);

    SoSeparator * root = new SoSeparator;
    simpleScene (root);

    // SoCone * cone = new SoCone;
    // root->addChild(cone);

    examiner->setSceneGraph(root);

    displayBranch (ui->tree->invisibleRootItem (), root);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::displayBranch (QTreeWidgetItem * target, SoNode * node)
{
    TreeItem * branch = new TreeItem;
    branch->node = node;

    SoType type = node->getTypeId ();
    branch->setText (0, type.getName().getString());

    if (type.isDerivedFrom (SoGroup::getClassTypeId ()) )
    {
       SoGroup * group = dynamic_cast <SoGroup *> (node);
       int count = group->getNumChildren ();
       for (int i = 0; i < count; i++)
       {
           SoNode * sub_node = group->getChild (i);
           displayBranch (branch, sub_node);
       }
    }

    target->addChild (branch);
}

void MainWindow::displayLine (QString name, QString value)
{
    QTreeWidgetItem * item = new QTreeWidgetItem (ui->prop);
    item->setText (0, name);
    item->setText (1, value);
}

void MainWindow::displayNode (SoNode * node)
{
   ui->prop->clear ();

   SoFieldContainer * obj = node;

   SoFieldList list;
   obj->getFields (list);
   int len = list.getLength ();

   // display fields
   for (int i = 0; i < len; i++)
   {
       SoField * field = list [i];

       // retrieve field name
       SbName name;
       obj->getFieldName (field, name);

       // retrieve field value
       SbString value;
       field->get (value);

       displayLine (name.getString (), value.getString ());
   }
}


void MainWindow::on_tree_itemActivated (QTreeWidgetItem * param, int column)
{
   TreeItem * item = dynamic_cast <TreeItem *> (param);
   if (item != nullptr)
      displayNode (item->node);
}


void MainWindow::simpleScene (SoSeparator * top)
{
    // cone
    SoSeparator * group1 = new SoSeparator;

    SoTransform * shift1 = new SoTransform;
    shift1->translation.setValue (-2.0, 0.0, 0.0);
    group1->addChild (shift1);

    SoMaterial * redMaterial = new SoMaterial;
    redMaterial->diffuseColor.setValue (1.0, 0.0, 0.0);
    group1->addChild (redMaterial);

    SoCone * cone = new SoCone ();
    group1->addChild (cone);

    top->addChild (group1);

    // sphere
    SoSeparator * group2 = new SoSeparator;

    SoMaterial * greenMaterial = new SoMaterial;
    greenMaterial->diffuseColor.setValue (0.0, 1.0, 0.0);
    group2->addChild (greenMaterial);

    SoSphere * sphere = new SoSphere ();
    group2->addChild (sphere);

    top->addChild (group2);

    // cube
    SoSeparator * group3 = new SoSeparator;

    SoTransform * shift3 = new SoTransform ();
    shift3->translation.setValue (1.6, 0.0, 1.6);
    shift3->scaleFactor.setValue (0.7, 0.7, 0.7);
    group3->addChild (shift3);

    SoMaterial * blueMaterial = new SoMaterial;
    blueMaterial->diffuseColor.setValue (0.0, 0.0, 1.0);
    group3->addChild (blueMaterial);

    SoCube * cube = new SoCube ();
    group3->addChild (cube);

    top->addChild (group3);

}



int main (int argc, char ** argv)
{
  QApplication app (argc, argv);

  SoQt::init ((QWidget*)NULL);

  MainWindow * win = new MainWindow();
  win->show();

  QObject::connect (&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));

  SoQt::mainLoop();

  return 0;
}

